<?php

namespace Drupal\bing_indexing_api\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class BulkUpdateForm.
 */
class BingBulkUpdateForm extends FormBase {

  /**
   * The Messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Class constructor.
   */
  public function __construct(MessengerInterface $messenger) {
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('messenger')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'bing_index_api_bulk_update_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $form['get'] = [
      '#type' => 'markup',
      '#markup' => '<h2>' . $this->t('Import your list below') . '</h2>',
    ];

    $form['urls'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Urls to upload'),
      '#states' => [
        'visible' => [':input[name="upload"]' => ['checked' => FALSE]],
      ],
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Submit'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);

    // Validate the form here instead of using required.
    $urls = $form_state->getValue('urls');

    if (empty($urls)) {
      $form_state->setErrorByName('urls', 'You can\'t leave this blank');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Process the URLS.
    $urls = $this->processUrls($form_state);
    foreach ($urls as $url) {
      $operations[] = [
        'Drupal\bing_indexing_api\Batch\BingIndexApiBatch::batchProcess',
        [$url],
      ];
    }

    // Fail safe in case the batch fails in building the operations.
    if (isset($operations)) {
      // Set the batch to win the stuff.
      $batch = [
        'title' => $this->t('Checking...'),
        'operations' => $operations,
        'init_message' => $this->t('Importing URLs to process.'),
        'finished' => 'Drupal\bing_indexing_api\Batch\BingIndexApiBatch::batchFinished',
        'file' => drupal_get_path('module', 'bing_indexing_api') . '/src/Batch/BingIndexApiBatch.php',
      ];

      // Engage.
      batch_set($batch);
    }
    else {
      $this->messenger->addMessage($this->t('No Valid URLs to Process!'));
    }
  }

  /**
   * Make all the urls happy via text or file.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Obvious form objects.
   *
   * @return mixed
   *   The parsed happiness of urls to win.
   */
  protected function processUrls(FormStateInterface $form_state) {
    // Grab the urls.
    $urls = $form_state->getValue('urls');

    // If the text box.
    $urls = preg_split("/\r\n|\n|\r/", $urls);

    // Put the URLs into an array and try to get them in the best
    // format possible for domination.
    $parsed_urls = [];
    foreach ($urls as $url) {
      $url = trim(preg_replace("/\s+/", " ", $url));
      if (filter_var($url, FILTER_VALIDATE_URL)) {
        $parsed_urls[] = parse_url($url, PHP_URL_PATH);
      }
    }
    return $parsed_urls;
  }

}
